/* pgg_pubkey_clone.c - Clones a PggPubkey object
 *      Copyright (C) 1999 Michael Roth <mroth@gnupg.org>
 *
 * This file is part of PGG (Privacy Guard Glue).
 *
 * PGG is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * PGG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */


#include <includes.h>
#include <pgg.h>
#include <pggdebug.h>
#include <pggpubkey.h>


#define pubkey		((PggPubkeyPtr)(_pubkey))


PggPubkey pgg_pubkey_clone(PggPubkey _pubkey, PggErrenv errenv)
{
    PggErrenv		local_errenv;
    PggPubkeyPtr	new_pubkey;
    
    PGG_STD_ASSERT_ARG(PggPubkey, pubkey, NULL);
    
    pgg_errenv_reset(local_errenv);
    
    if (!( new_pubkey = _malloc(PggPubkey) ))
        PGG_RETURN_ERR_ARG(RESOURCE, MEMORY, NULL);
    
    memset(new_pubkey, 0, _size(PggPubkey));
    
    new_pubkey->magic      = PggPubkeyMAGIC;
    new_pubkey->refcounter = 1;
    
    if (pubkey->config)
        new_pubkey->config = pgg_config_clone(pubkey->config, local_errenv);
    
    if (pgg_errenv_is_set(local_errenv)) {
        pgg_config_release(new_pubkey->config, NULL);
        free(new_pubkey);
        
        pgg_errenv_copy(errenv, local_errenv);
        return NULL;
    }
    
    /* FIXME: Handle linked list of UIDs, subkeys, etc. !!! */
    
    return _hd(PggPubkey, pubkey);
}


